#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "configdialog.h"
#include "uploaddialog.h"
#include "aboutdialog.h"

#include <QSettings>
#include <QTextCharFormat>
#include <QFileDialog>

#include "termios.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    port = new QextSerialPort;
    ui->terminalLog->setPlainText("");
    timer = new QTimer;
    timer->start(50);
    connect(timer, SIGNAL(timeout()), this, SLOT(timeout()));

    dirLabel = new QLabel;
    ui->statusBar->addPermanentWidget(dirLabel, 100);

    QSettings settings("Andrey Demenev", "FlashTerm");
    settings.beginGroup("Upload");
    workingDir = settings.value("workingDir").toString();
    dirLabel->setText(workingDir);

}

MainWindow::~MainWindow()
{
    delete ui;
    delete port;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::on_actionConfig_activated()
{
    ConfigDialog *dlg = new ConfigDialog(this);
    dlg->exec();
    delete dlg;
}

void MainWindow::on_actionConnect_triggered(bool checked)
{
    port->setQueryMode(QextSerialBase::EventDriven);
    if (!checked) {
        port->close();
        return;
    }
    bool ok;
    QSettings settings("Andrey Demenev", "FlashTerm");
    settings.beginGroup("Terminal");
    port->setPortName(settings.value("Port").toString());

    do {
        if (!port->open(QIODevice::ReadWrite | QIODevice::Unbuffered)) {
            port->close();
            break;
        }

        uint d;

        BaudRateType baudRate;
        d = settings.value("BaudRate").toString().toUInt(&ok, 10);
        if (!ok) break;

        switch (d) {
            case 50: baudRate = BAUD50; break;
            case 75: baudRate = BAUD75; break;
            case 110: baudRate = BAUD110; break;
            case 134: baudRate = BAUD134; break;
            case 150: baudRate = BAUD150; break;
            case 200: baudRate = BAUD200; break;
            case 300: baudRate = BAUD300; break;
            case 600: baudRate = BAUD600; break;
            case 1200: baudRate = BAUD1200; break;
            case 1800: baudRate = BAUD1800; break;
            case 2400: baudRate = BAUD2400; break;
            case 4800: baudRate = BAUD4800; break;
            case 9600: baudRate = BAUD9600; break;
            case 14400: baudRate = BAUD14400; break;
            case 19200: baudRate = BAUD19200; break;
            case 38400: baudRate = BAUD38400; break;
            case 56000: baudRate = BAUD56000; break;
            case 57600: baudRate = BAUD57600; break;
            case 76800: baudRate = BAUD76800; break;
            case 115200: baudRate = BAUD115200; break;
            case 128000: baudRate = BAUD128000; break;
            case 256000: baudRate = BAUD256000; break;
            default: ok = false;
        }
        if (!ok) break;

        port->setBaudRate(baudRate);

        DataBitsType dataBits;
        d = settings.value("DataBits").toString().toUInt(&ok, 10);
        if (!ok) break;
        dataBits = DataBitsType(-1);
        switch (d) {
        case 5:
            dataBits = DATA_5;
            break;
        case 6:
            dataBits = DATA_6;
            break;
        case 7:
            dataBits = DATA_7;
            break;
        case 8:
            dataBits = DATA_8;
            break;
        }
        if (dataBits == DataBitsType(-1)) {
            break;
        }
        port->setDataBits(dataBits);

        d = settings.value("StopBits").toString().toUInt(&ok, 10);
        if (!ok) break;
        StopBitsType stopBits = (StopBitsType)-1;
        switch (d) {
        case 1:
            stopBits = STOP_1;
            break;
        case 2:
            stopBits = STOP_2;
            break;
        }
        if (stopBits == (StopBitsType)-1) {
            break;
        }
        port->setStopBits(stopBits);
        port->setFlowControl(FLOW_OFF);
        port->setTimeout(0);
        return;
    } while (0);

    port->close();
    ui->actionConnect->setChecked(false);
    ConfigDialog *dlg = new ConfigDialog(this);
    dlg->exec();
    delete dlg;
}

void MainWindow::on_terminalInput_returnPressed()
{
    QTextCharFormat fmt;
    fmt.setForeground(QBrush(QColor(255,0,0),Qt::SolidPattern));
    ui->terminalLog->setCurrentCharFormat(fmt);
    QString str = ui->terminalInput->text() + "\n";
    qDebug("%s", str.toAscii().data());
    port->write(str.toAscii());
    ui->terminalLog->textCursor().movePosition(QTextCursor::End);
    ui->terminalLog->insertPlainText(str);
    ui->terminalLog->ensureCursorVisible();
    ui->terminalInput->setText("");
}

void MainWindow::on_toolButton_clicked()
{
    on_terminalInput_returnPressed();
}

void MainWindow::timeout()
{
    qint64 n = port->bytesAvailable();
    if (n > 0) {
        QTextCharFormat fmt;
        fmt.setForeground(QBrush(QColor(0,0,255),Qt::SolidPattern));
        ui->terminalLog->setCurrentCharFormat(fmt);
        QByteArray data = port->readAll();
        ui->terminalLog->textCursor().movePosition(QTextCursor::End);
        ui->terminalLog->insertPlainText(QString(data));
        ui->terminalLog->ensureCursorVisible();
    }
}

void MainWindow::on_actionSelect_directory_triggered()
{
    QString dir = QFileDialog::getExistingDirectory(this, tr("Select directory with makefile"));
    if (dir == "") return;
    workingDir = dir;
    dirLabel->setText(dir);
    QSettings settings("Andrey Demenev", "FlashTerm");
    settings.beginGroup("Upload");
    settings.setValue("workingDir", workingDir);
    dirLabel->setText(workingDir);
}

void MainWindow::on_actionUpload_triggered()
{
    if (workingDir == "") {
        on_actionSelect_directory_triggered();
        if (workingDir == "") {
            return;
        }
    }
    bool open = port->isOpen();
    if (open) port->close();
    UploadDialog *u = new UploadDialog(this);
    u->upload(workingDir);
    delete u;
    if (open) port->open(QIODevice::ReadWrite | QIODevice::Unbuffered);
}

void MainWindow::on_actionClear_triggered()
{
    ui->terminalLog->clear();
}

void MainWindow::on_actionAbout_triggered()
{
    AboutDialog about(this);
    about.exec();
}

void MainWindow::on_actionAbout_QT_triggered()
{
    QApplication::aboutQt();
}
